@@ -339,7 +339,7 @@ arv_device_set_boolean_feature_value (ArvDevice *device, const char *feature, gb
 
 	node = arv_device_get_feature (device, feature);
 
-	if (ARV_IS_GC_INTEGER (node))
+	if (ARV_IS_GC_BOOLEAN (node))
 		arv_gc_boolean_set_value (ARV_GC_BOOLEAN (node), value, &error);
 	else
 		arv_warning_device ("[ArvDevice::set_boolean_feature_value] Node '%s' is not a boolean",
@@ -486,6 +486,16 @@ arv_device_get_integer_feature_value (ArvDevice *device, const char *feature)
 	return value;
 }
 
+/**
+ * arv_device_get_integer_feature_bounds:
+ * @device: a #ArvDevice
+ * @feature: feature name
+ * @min: (out): minimum feature value
+ * @max: (out): maximum feature value
+ *
+ * Retrieves feature bounds.
+ */
+
 void
 arv_device_get_integer_feature_bounds (ArvDevice *device, const char *feature, gint64 *min, gint64 *max)
 {
@@ -580,6 +590,16 @@ arv_device_get_float_feature_value (ArvDevice *device, const char *feature)
 	return value;
 }
 
+/**
+ * arv_device_get_float_feature_bounds:
+ * @device: a #ArvDevice
+ * @feature: feature name
+ * @min: (out): minimum feature value
+ * @max: (out): maximum feature value
+ *
+ * Retrieves feature bounds.
+ */
+
 void
 arv_device_get_float_feature_bounds (ArvDevice *device, const char *feature, double *min, double *max)
 {
@@ -747,7 +767,9 @@ arv_device_class_init (ArvDeviceClass *device_class)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (device_class);
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 	g_type_class_add_private (device_class, sizeof (ArvDevicePrivate));
+#endif
 
 	parent_class = g_type_class_peek_parent (device_class);
 
@@ -774,4 +796,8 @@ arv_device_class_init (ArvDeviceClass *device_class)
 			      g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0, G_TYPE_NONE);
 }
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 G_DEFINE_ABSTRACT_TYPE (ArvDevice, arv_device, G_TYPE_OBJECT)
+#else
+G_DEFINE_ABSTRACT_TYPE_WITH_CODE (ArvDevice, arv_device, G_TYPE_OBJECT, G_ADD_PRIVATE (ArvDevice))
+#endif
