@@ -33,6 +33,7 @@
 #include <arvgcnode.h>
 #include <arvgcpropertynode.h>
 #include <arvgcindexnode.h>
+#include <arvgcvalueindexednode.h>
 #include <arvgcinvalidatornode.h>
 #include <arvgcregisterdescriptionnode.h>
 #include <arvgcgroupnode.h>
@@ -172,6 +173,14 @@ arv_gc_create_element (ArvDomDocument *document, const char *tag_name)
 		node = arv_gc_property_node_new_p_port ();
 	else if (strcmp (tag_name, "pVariable") == 0)
 		node = arv_gc_property_node_new_p_variable ();
+	else if (strcmp (tag_name, "ValueIndexed") == 0)
+		node = arv_gc_value_indexed_node_new ();
+	else if (strcmp (tag_name, "pValueIndexed") == 0)
+		node = arv_gc_p_value_indexed_node_new ();
+	else if (strcmp (tag_name, "ValueDefault") == 0)
+		node = arv_gc_property_node_new_value_default ();
+	else if (strcmp (tag_name, "pValueDefault") == 0)
+		node = arv_gc_property_node_new_p_value_default ();
 	else if (strcmp (tag_name, "Formula") == 0)
 		node = arv_gc_property_node_new_formula ();
 	else if (strcmp (tag_name, "FormulaTo") == 0)
@@ -377,7 +386,9 @@ arv_gc_class_init (ArvGcClass *node_class)
 	ArvDomNodeClass *d_node_class = ARV_DOM_NODE_CLASS (node_class);
 	ArvDomDocumentClass *d_document_class = ARV_DOM_DOCUMENT_CLASS (node_class);
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 	g_type_class_add_private (node_class, sizeof (ArvGcPrivate));
+#endif
 
 	parent_class = g_type_class_peek_parent (node_class);
 
@@ -386,4 +397,8 @@ arv_gc_class_init (ArvGcClass *node_class)
 	d_document_class->create_element = arv_gc_create_element;
 }
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 G_DEFINE_TYPE (ArvGc, arv_gc, ARV_TYPE_DOM_DOCUMENT)
+#else
+G_DEFINE_TYPE_WITH_CODE (ArvGc, arv_gc, ARV_TYPE_DOM_DOCUMENT, G_ADD_PRIVATE (ArvGc))
+#endif
