@@ -44,7 +44,6 @@
 #ifdef ARAVIS_BUILD_PACKET_SOCKET
 #include <ifaddrs.h>
 #include <netinet/udp.h>
-#include <cap-ng.h>
 #include <net/if.h>
 #include <netinet/in.h>
 #include <linux/if_packet.h>
@@ -60,6 +59,7 @@
 #define ARV_GV_STREAM_POLL_TIMEOUT_US			1000000
 #define ARV_GV_STREAM_PACKET_TIMEOUT_US_DEFAULT		40000
 #define ARV_GV_STREAM_FRAME_RETENTION_US_DEFAULT	200000
+#define ARV_GV_STREAM_PACKET_REQUEST_RATIO_DEFAULT	0.10
 
 #define ARV_GV_STREAM_DISCARD_LATE_FRAME_THRESHOLD	100
 
@@ -68,6 +68,7 @@ enum {
 	ARV_GV_STREAM_PROPERTY_SOCKET_BUFFER,
 	ARV_GV_STREAM_PROPERTY_SOCKET_BUFFER_SIZE,
 	ARV_GV_STREAM_PROPERTY_PACKET_RESEND,
+	ARV_GV_STREAM_PROPERTY_PACKET_REQUEST_RATIO,
 	ARV_GV_STREAM_PROPERTY_PACKET_TIMEOUT,
 	ARV_GV_STREAM_PROPERTY_FRAME_RETENTION
 } ArvGvStreamProperties;
@@ -118,6 +119,7 @@ struct _ArvGvStreamThreadData {
 	guint16 stream_port;
 
 	ArvGvStreamPacketResend packet_resend;
+	double packet_request_ratio;
 	guint packet_timeout_us;
 	guint frame_retention_us;
 
@@ -237,11 +239,12 @@ _process_data_leader (ArvGvStreamThreadData *thread_data,
 		return;
 	}
 
-	frame->buffer->priv->gvsp_payload_type = arv_gvsp_packet_get_payload_type (packet);
+	frame->buffer->priv->payload_type = arv_gvsp_packet_get_buffer_payload_type (packet);
 	frame->buffer->priv->frame_id = arv_gvsp_packet_get_frame_id (packet);
+	frame->buffer->priv->chunk_endianness = G_BIG_ENDIAN;
 
 	frame->buffer->priv->system_timestamp_ns = g_get_real_time() * 1000LL;
-	if (frame->buffer->priv->gvsp_payload_type != ARV_GVSP_PAYLOAD_TYPE_H264) {
+	if (frame->buffer->priv->payload_type != ARV_BUFFER_PAYLOAD_TYPE_H264) {
 		if (G_LIKELY (thread_data->timestamp_tick_frequency != 0))
 			frame->buffer->priv->timestamp_ns = arv_gvsp_packet_get_timestamp (packet,
 											   thread_data->timestamp_tick_frequency);
@@ -251,7 +254,8 @@ _process_data_leader (ArvGvStreamThreadData *thread_data,
 	} else
 		frame->buffer->priv->timestamp_ns = frame->buffer->priv->system_timestamp_ns;
 
-	if (frame->buffer->priv->gvsp_payload_type == ARV_GVSP_PAYLOAD_TYPE_IMAGE) {
+	if (frame->buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_IMAGE ||
+	    frame->buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA) {
 		frame->buffer->priv->x_offset = arv_gvsp_packet_get_x_offset (packet);
 		frame->buffer->priv->y_offset = arv_gvsp_packet_get_y_offset (packet);
 		frame->buffer->priv->width = arv_gvsp_packet_get_width (packet);
@@ -415,12 +419,16 @@ _missing_packet_check (ArvGvStreamThreadData *thread_data,
 		       guint32 packet_id,
 		       guint64 time_us)
 {
+	guint n_packet_requests = 0;
 	int i;
 
 	if (thread_data->packet_resend == ARV_GV_STREAM_PACKET_RESEND_NEVER ||
 	    frame->error_packet_received)
 		return;
 
+	if ((int) (frame->n_packets * thread_data->packet_request_ratio) <= 0)
+		return;
+
 	if (packet_id < frame->n_packets) {
 		int first_missing = -1;
 
@@ -434,6 +442,19 @@ _missing_packet_check (ArvGvStreamThreadData *thread_data,
 				if (first_missing >= 0) {
 					int j;
 
+					if (i - first_missing + n_packet_requests >
+					    (frame->n_packets * thread_data->packet_request_ratio)) {
+						n_packet_requests += i - first_missing;
+
+						arv_log_stream_thread ("[GvStream::missing_packet_check]"
+									 " Maximum number of packet requests "
+									 "reached at dt = %" G_GINT64_FORMAT ", n_requests = %u/%u",
+									 time_us - frame->first_packet_time_us,
+									 n_packet_requests, frame->n_packets);
+
+						return;
+					}
+
 					arv_log_stream_thread ("[GvStream::missing_packet_check]"
 							       " Resend request at dt = %" G_GINT64_FORMAT ", packet id = %u/%u",
 							       time_us - frame->first_packet_time_us,
@@ -452,6 +473,20 @@ _missing_packet_check (ArvGvStreamThreadData *thread_data,
 		if (first_missing >= 0) {
 			int j;
 
+			if (i - first_missing + n_packet_requests >
+			    (frame->n_packets * thread_data->packet_request_ratio)) {
+				n_packet_requests += i - first_missing;
+
+				arv_log_stream_thread ("[GvStream::missing_packet_check]"
+						       " Maximum number of packet requests "
+						       "reached at dt = %" G_GINT64_FORMAT ", n_requests = %u/%u",
+						       time_us - frame->first_packet_time_us,
+						       n_packet_requests, frame->n_packets);
+
+				return;
+			}
+
+
 			arv_log_stream_thread ("[GvStream::missing_packet_check]"
 					       " Resend request at dt = %" G_GINT64_FORMAT", packet id = %u/%u",
 					       time_us - frame->first_packet_time_us,
@@ -654,6 +689,8 @@ _process_packet (ArvGvStreamThreadData *thread_data, const ArvGvspPacket *packet
 					break;
 			frame->last_valid_packet = i - 1;
 
+			arv_gvsp_packet_debug (packet, packet_size, ARV_DEBUG_LEVEL_LOG);
+
 			switch (arv_gvsp_packet_get_content_type (packet)) {
 				case ARV_GVSP_CONTENT_TYPE_DATA_LEADER:
 					_process_data_leader (thread_data, frame, packet, packet_id);
@@ -917,6 +954,9 @@ static void *
 arv_gv_stream_thread (void *data)
 {
 	ArvGvStreamThreadData *thread_data = data;
+#ifdef ARAVIS_BUILD_PACKET_SOCKET
+	int fd;
+#endif
 
 	thread_data->frames = NULL;
 	thread_data->last_frame_id = 0;
@@ -931,9 +971,10 @@ arv_gv_stream_thread (void *data)
 		thread_data->callback (thread_data->user_data, ARV_STREAM_CALLBACK_TYPE_INIT, NULL);
 
 #ifdef ARAVIS_BUILD_PACKET_SOCKET
-	if (capng_have_capability(CAPNG_EFFECTIVE, CAP_NET_RAW) && thread_data->use_packet_socket)
+	if (thread_data->use_packet_socket && (fd = socket (PF_PACKET, SOCK_RAW, g_htons (ETH_P_ALL))) >= 0) {
+		close (fd);
 		_ring_buffer_loop (thread_data);
-	else
+	} else
 #endif
 		_loop (thread_data);
 
@@ -1010,6 +1051,7 @@ arv_gv_stream_new (ArvGvDevice *gv_device,
 	thread_data->callback = callback;
 	thread_data->user_data = user_data;
 	thread_data->packet_resend = ARV_GV_STREAM_PACKET_RESEND_ALWAYS;
+	thread_data->packet_request_ratio = ARV_GV_STREAM_PACKET_REQUEST_RATIO_DEFAULT;
 	thread_data->packet_timeout_us = ARV_GV_STREAM_PACKET_TIMEOUT_US_DEFAULT;
 	thread_data->frame_retention_us = ARV_GV_STREAM_FRAME_RETENTION_US_DEFAULT;
 	thread_data->timestamp_tick_frequency = timestamp_tick_frequency;
@@ -1055,7 +1097,7 @@ arv_gv_stream_new (ArvGvDevice *gv_device,
 	thread_data->interface_address = g_object_ref (interface_address);
 	thread_data->interface_socket_address = g_inet_socket_address_new (interface_address, 0);
 	thread_data->device_socket_address = g_inet_socket_address_new (device_address, ARV_GVCP_PORT);
-	g_socket_bind (thread_data->socket, thread_data->interface_socket_address, TRUE, NULL);
+	g_socket_bind (thread_data->socket, thread_data->interface_socket_address, FALSE, NULL);
 
 	local_address = G_INET_SOCKET_ADDRESS (g_socket_get_local_address (thread_data->socket, NULL));
 	thread_data->stream_port = g_inet_socket_address_get_port (local_address);
@@ -1069,7 +1111,7 @@ arv_gv_stream_new (ArvGvDevice *gv_device,
 	arv_debug_stream ("[GvStream::stream_new] Destination stream port = %d", thread_data->stream_port);
 	arv_debug_stream ("[GvStream::stream_new] Source stream port = %d", thread_data->source_stream_port);
 
-	gv_stream->priv->thread = arv_g_thread_new ("arv_gv_stream", arv_gv_stream_thread, gv_stream->priv->thread_data);
+	gv_stream->priv->thread = g_thread_new ("arv_gv_stream", arv_gv_stream_thread, gv_stream->priv->thread_data);
 
 	return ARV_STREAM (gv_stream);
 }
@@ -1129,6 +1171,9 @@ arv_gv_stream_set_property (GObject * object, guint prop_id,
 		case ARV_GV_STREAM_PROPERTY_PACKET_RESEND:
 			thread_data->packet_resend = g_value_get_enum (value);
 			break;
+		case ARV_GV_STREAM_PROPERTY_PACKET_REQUEST_RATIO:
+			thread_data->packet_request_ratio = g_value_get_double (value);
+			break;
 		case ARV_GV_STREAM_PROPERTY_PACKET_TIMEOUT:
 			thread_data->packet_timeout_us = g_value_get_uint (value);
 			break;
@@ -1160,6 +1205,9 @@ arv_gv_stream_get_property (GObject * object, guint prop_id,
 		case ARV_GV_STREAM_PROPERTY_PACKET_RESEND:
 			g_value_set_enum (value, thread_data->packet_resend);
 			break;
+		case ARV_GV_STREAM_PROPERTY_PACKET_REQUEST_RATIO:
+			g_value_set_double (value, thread_data->packet_request_ratio);
+			break;
 		case ARV_GV_STREAM_PROPERTY_PACKET_TIMEOUT:
 			g_value_set_uint (value, thread_data->packet_timeout_us);
 			break;
@@ -1251,7 +1299,9 @@ arv_gv_stream_class_init (ArvGvStreamClass *gv_stream_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (gv_stream_class);
 	ArvStreamClass *stream_class = ARV_STREAM_CLASS (gv_stream_class);
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 	g_type_class_add_private (gv_stream_class, sizeof (ArvGvStreamPrivate));
+#endif
 
 	parent_class = g_type_class_peek_parent (gv_stream_class);
 
@@ -1284,6 +1334,13 @@ arv_gv_stream_class_init (ArvGvStreamClass *gv_stream_class)
 				   ARV_GV_STREAM_PACKET_RESEND_ALWAYS,
 				   G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)
 		);
+	g_object_class_install_property (
+		object_class, ARV_GV_STREAM_PROPERTY_PACKET_REQUEST_RATIO,
+		g_param_spec_double ("packet-request-ratio", "Packet request ratio",
+				     "Packet resend request limit as a percentage of frame packet number",
+				     0.0, 2.0, ARV_GV_STREAM_PACKET_REQUEST_RATIO_DEFAULT,
+				     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)
+		);
 	g_object_class_install_property (
 		object_class, ARV_GV_STREAM_PROPERTY_PACKET_TIMEOUT,
 		g_param_spec_uint ("packet-timeout", "Packet timeout",
@@ -1304,4 +1361,8 @@ arv_gv_stream_class_init (ArvGvStreamClass *gv_stream_class)
 		);
 }
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 G_DEFINE_TYPE (ArvGvStream, arv_gv_stream, ARV_TYPE_STREAM)
+#else
+G_DEFINE_TYPE_WITH_CODE (ArvGvStream, arv_gv_stream, ARV_TYPE_STREAM, G_ADD_PRIVATE (ArvGvStream))
+#endif
