@@ -159,7 +159,7 @@ _usb_device_to_device_ids (ArvUvInterface *uv_interface, libusb_device *device)
 
 	r = libusb_get_device_descriptor (device, &desc);
 	if (r < 0) {
-		g_warning ("Failed to get device descriptor");
+		arv_warning_interface ("Failed to get device descriptor");
 		return NULL;
 	}
 
@@ -330,7 +330,7 @@ arv_uv_interface_open_device (ArvInterface *interface, const char *device_id)
 }
 
 static ArvInterface *uv_interface = NULL;
-ARV_DEFINE_STATIC_MUTEX (uv_interface_mutex);
+static GMutex uv_interface_mutex;
 
 /**
  * arv_uv_interface_get_instance:
@@ -343,12 +343,12 @@ ARV_DEFINE_STATIC_MUTEX (uv_interface_mutex);
 ArvInterface *
 arv_uv_interface_get_instance (void)
 {
-	arv_g_mutex_lock (&uv_interface_mutex);
+	g_mutex_lock (&uv_interface_mutex);
 
 	if (uv_interface == NULL)
 		uv_interface = g_object_new (ARV_TYPE_UV_INTERFACE, NULL);
 
-	arv_g_mutex_unlock (&uv_interface_mutex);
+	g_mutex_unlock (&uv_interface_mutex);
 
 	return ARV_INTERFACE (uv_interface);
 }
@@ -356,11 +356,11 @@ arv_uv_interface_get_instance (void)
 void
 arv_uv_interface_destroy_instance (void)
 {
-	arv_g_mutex_lock (&uv_interface_mutex);
+	g_mutex_lock (&uv_interface_mutex);
 
 	g_clear_object (&uv_interface);
 
-	arv_g_mutex_unlock (&uv_interface_mutex);
+	g_mutex_unlock (&uv_interface_mutex);
 }
 
 static void
@@ -391,7 +391,9 @@ arv_uv_interface_class_init (ArvUvInterfaceClass *uv_interface_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (uv_interface_class);
 	ArvInterfaceClass *interface_class = ARV_INTERFACE_CLASS (uv_interface_class);
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 	g_type_class_add_private (uv_interface_class, sizeof (ArvUvInterfacePrivate));
+#endif
 
 	parent_class = g_type_class_peek_parent (uv_interface_class);
 
@@ -403,4 +405,8 @@ arv_uv_interface_class_init (ArvUvInterfaceClass *uv_interface_class)
 	interface_class->protocol = "USB3Vision";
 }
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 G_DEFINE_TYPE (ArvUvInterface, arv_uv_interface, ARV_TYPE_INTERFACE)
+#else
+G_DEFINE_TYPE_WITH_CODE (ArvUvInterface, arv_uv_interface, ARV_TYPE_INTERFACE, G_ADD_PRIVATE (ArvUvInterface))
+#endif
