@@ -138,13 +138,17 @@ arv_uv_stream_thread (void *data)
 					if (buffer != NULL) {
 						buffer->priv->system_timestamp_ns = g_get_real_time () * 1000LL;
 						buffer->priv->status = ARV_BUFFER_STATUS_FILLING;
-						buffer->priv->gvsp_payload_type = ARV_GVSP_PAYLOAD_TYPE_IMAGE;
-						arv_uvsp_packet_get_region (packet,
-									    &buffer->priv->width,
-									    &buffer->priv->height,
-									    &buffer->priv->x_offset,
-									    &buffer->priv->y_offset);
-						buffer->priv->pixel_format = arv_uvsp_packet_get_pixel_format (packet);
+						buffer->priv->payload_type = arv_uvsp_packet_get_buffer_payload_type (packet);
+						buffer->priv->chunk_endianness = G_LITTLE_ENDIAN;
+						if (buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_IMAGE ||
+						    buffer->priv->payload_type == ARV_BUFFER_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA) {
+							arv_uvsp_packet_get_region (packet,
+										    &buffer->priv->width,
+										    &buffer->priv->height,
+										    &buffer->priv->x_offset,
+										    &buffer->priv->y_offset);
+							buffer->priv->pixel_format = arv_uvsp_packet_get_pixel_format (packet);
+						}
 						buffer->priv->frame_id = arv_uvsp_packet_get_frame_id (packet);
 						buffer->priv->timestamp_ns = arv_uvsp_packet_get_timestamp (packet);
 						offset = 0;
@@ -207,13 +211,24 @@ arv_uv_stream_thread (void *data)
 
 	g_free (incoming_buffer);
 
+        /* The thread was cancelled with unprocessed frame. Release it to prevent memory leak */
+        if (buffer != NULL)
+       		g_object_unref(buffer);
+
 	arv_log_stream_thread ("Stop USB3Vision stream thread");
 
 	return NULL;
 }
 
-/* ArvUvStream implemenation */
+/* ArvUvStream implementation */
 
+static guint32
+align (guint32 val, guint32 alignment)
+{
+	g_assert (alignment % 2 == 0 && alignment > 0);
+
+	return (val + (alignment - 1)) & ~(alignment - 1);
+}
 
 /**
  * arv_uv_stream_new: (skip)
@@ -233,6 +248,7 @@ arv_uv_stream_new (ArvUvDevice *uv_device, ArvStreamCallback callback, void *use
 	ArvStream *stream;
 	guint64 offset;
 	guint64 sirm_offset;
+	guint32 si_info;
 	guint64 si_req_payload_size;
 	guint32 si_req_leader_size;
 	guint32 si_req_trailer_size;
@@ -241,6 +257,8 @@ arv_uv_stream_new (ArvUvDevice *uv_device, ArvStreamCallback callback, void *use
 	guint32 si_transfer1_size;
 	guint32 si_transfer2_size;
 	guint32 si_control;
+	guint32 alignment;
+	guint32 aligned_maximum_transfer_size;
 
 	g_return_val_if_fail (ARV_IS_UV_DEVICE (uv_device), NULL);
 
@@ -248,27 +266,39 @@ arv_uv_stream_new (ArvUvDevice *uv_device, ArvStreamCallback callback, void *use
 
 	arv_device_read_memory (device, ARV_ABRM_SBRM_ADDRESS, sizeof (guint64), &offset, NULL);
 	arv_device_read_memory (device, offset + ARV_SBRM_SIRM_ADDRESS, sizeof (guint64), &sirm_offset, NULL);
+	arv_device_read_memory (device, sirm_offset + ARV_SI_INFO, sizeof (si_info), &si_info, NULL);
 	arv_device_read_memory (device, sirm_offset + ARV_SI_REQ_PAYLOAD_SIZE, sizeof (si_req_payload_size), &si_req_payload_size, NULL);
 	arv_device_read_memory (device, sirm_offset + ARV_SI_REQ_LEADER_SIZE, sizeof (si_req_leader_size), &si_req_leader_size, NULL);
 	arv_device_read_memory (device, sirm_offset + ARV_SI_REQ_TRAILER_SIZE, sizeof (si_req_trailer_size), &si_req_trailer_size, NULL);
 
+	alignment = 1 << ((si_info & ARV_SI_INFO_ALIGNMENT_MASK) >> ARV_SI_INFO_ALIGNMENT_SHIFT);
+
+	arv_debug_stream ("SI_INFO            =       0x%08x", si_info);
 	arv_debug_stream ("SI_REQ_PAYLOAD_SIZE =      0x%016lx", si_req_payload_size);
 	arv_debug_stream ("SI_REQ_LEADER_SIZE =       0x%08x", si_req_leader_size);
 	arv_debug_stream ("SI_REQ_TRAILER_SIZE =      0x%08x", si_req_trailer_size);
 
+	arv_debug_stream ("Required alignment =       %d", alignment);
+
+	aligned_maximum_transfer_size = ARV_UV_STREAM_MAXIMUM_TRANSFER_SIZE / alignment * alignment;
+
 	if (si_req_leader_size < 1) {
-		arv_warning_stream ("Wrong SI_REQ_LEADER_SIZE value, using %d instead", ARV_UV_STREAM_MAXIMUM_TRANSFER_SIZE);
-		si_req_leader_size = ARV_UV_STREAM_MAXIMUM_TRANSFER_SIZE;
+		arv_warning_stream ("Wrong SI_REQ_LEADER_SIZE value, using %d instead", aligned_maximum_transfer_size);
+		si_req_leader_size = aligned_maximum_transfer_size;
+	} else {
+		si_req_leader_size = align (si_req_leader_size, alignment);
 	}
 
 	if (si_req_trailer_size < 1) {
-		arv_warning_stream ("Wrong SI_REQ_TRAILER_SIZE value, using %d instead", ARV_UV_STREAM_MAXIMUM_TRANSFER_SIZE);
-		si_req_trailer_size = ARV_UV_STREAM_MAXIMUM_TRANSFER_SIZE;
+		arv_warning_stream ("Wrong SI_REQ_TRAILER_SIZE value, using %d instead", aligned_maximum_transfer_size);
+		si_req_trailer_size = aligned_maximum_transfer_size;
+	} else {
+		si_req_trailer_size = align (si_req_trailer_size, alignment);
 	}
 
-	si_payload_size = ARV_UV_STREAM_MAXIMUM_TRANSFER_SIZE;
+	si_payload_size = aligned_maximum_transfer_size;
 	si_payload_count=  si_req_payload_size / si_payload_size;
-	si_transfer1_size = si_req_payload_size % si_payload_size;
+	si_transfer1_size = align(si_req_payload_size % si_payload_size, alignment);
 	si_transfer2_size = 0;
 
 	arv_device_write_memory (device, sirm_offset + ARV_SI_MAX_LEADER_SIZE, sizeof (si_req_leader_size), &si_req_leader_size, NULL);
@@ -308,7 +338,7 @@ arv_uv_stream_new (ArvUvDevice *uv_device, ArvStreamCallback callback, void *use
 	thread_data->n_underruns = 0;
 
 	uv_stream->priv->thread_data = thread_data;
-	uv_stream->priv->thread = arv_g_thread_new ("arv_uv_stream", arv_uv_stream_thread, uv_stream->priv->thread_data);
+	uv_stream->priv->thread = g_thread_new ("arv_uv_stream", arv_uv_stream_thread, uv_stream->priv->thread_data);
 
 	return ARV_STREAM (uv_stream);
 }
@@ -384,7 +414,9 @@ arv_uv_stream_class_init (ArvUvStreamClass *uv_stream_class)
 	GObjectClass *object_class = G_OBJECT_CLASS (uv_stream_class);
 	ArvStreamClass *stream_class = ARV_STREAM_CLASS (uv_stream_class);
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 	g_type_class_add_private (uv_stream_class, sizeof (ArvUvStreamPrivate));
+#endif
 
 	parent_class = g_type_class_peek_parent (uv_stream_class);
 
@@ -393,4 +425,8 @@ arv_uv_stream_class_init (ArvUvStreamClass *uv_stream_class)
 	stream_class->get_statistics = arv_uv_stream_get_statistics;
 }
 
+#if !GLIB_CHECK_VERSION(2,38,0)
 G_DEFINE_TYPE (ArvUvStream, arv_uv_stream, ARV_TYPE_STREAM)
+#else
+G_DEFINE_TYPE_WITH_CODE (ArvUvStream, arv_uv_stream, ARV_TYPE_STREAM, G_ADD_PRIVATE (ArvUvStream))
+#endif
